from celery import Celery
from kombu.common import Broadcast

from django.conf import settings
from django.db import connection


def notify_agents(data):
    '''Send notifications to all other tenants'''
    tenant = connection.get_tenant()
    if not hasattr(tenant, 'domain_url'):
        # Fake tenant, certainly during migration, do nothing
        return
    notification = {
        'tenant': tenant.domain_url,
        'data': data,
    }
    with Celery('hobo', broker=settings.BROKER_URL) as app:
        app.conf.update(
            CELERY_TASK_SERIALIZER='json',
            CELERY_ACCEPT_CONTENT=['json'],
            CELERY_RESULT_SERIALIZER='json',
            CELERY_QUEUES=(Broadcast('broadcast_tasks'), ),
            CELERY_EVENT_QUEUE_TTL=10,
        )
        # see called method in hobo.agent.worker.celery
        app.send_task('hobo-notify',
                      (notification,),
                      expires=settings.BROKER_TASK_EXPIRES,
                      queue='broadcast_tasks')
