from django.conf import settings
from django.utils.deprecation import MiddlewareMixin


class XForwardedForMiddleware(MiddlewareMixin):
    '''Copy the first address from X-Forwarded-For header to the REMOTE_ADDR meta.

       This middleware should only be used if you are sure the header cannot be
       forged (behind a reverse proxy for example).'''
    def process_request(self, request):
        if getattr(settings, 'USE_X_FORWARDED_FOR', False):
            if 'HTTP_X_FORWARDED_FOR' in request.META:
                ip =  request.META.get('HTTP_X_FORWARDED_FOR', '').split(",")[0].strip()
                if ip:
                    request.META['REMOTE_ADDR'] = ip
        return None

