import os

from mock import mock_open, patch

from django.utils.six.moves import builtins

# Debian defaults
DEBUG = False

PROJECT_NAME = 'authentic2-multitenant'


open_backup = open
with patch.object(builtins, 'open', mock_open(read_data=b'xxx')):
    exec(open_backup(os.environ['DEBIAN_CONFIG_COMMON']).read())

BRANCH_NAME = os.environ.get("BRANCH_NAME", "").replace('/', '-')
DATABASES['default']['TEST'] = {
    'NAME': DATABASES['default']['NAME'] + '-%s' % BRANCH_NAME,
}

# Avoid conflic with real tenants
# that might exist in /var/lib/authentic2_multitenant/tenants
TENANT_BASE = '/that/path/does/not/exist'


# Add the XForwardedForMiddleware
MIDDLEWARE = ('authentic2.middleware.XForwardedForMiddleware',) + MIDDLEWARE

# Add authentic settings loader
TENANT_SETTINGS_LOADERS = ('hobo.multitenant.settings_loaders.Authentic',) + TENANT_SETTINGS_LOADERS

# Add authentic2 hobo agent
INSTALLED_APPS = ('hobo.agent.authentic2',) + INSTALLED_APPS

CACHES = {
    'default': {
            'BACKEND': 'hobo.multitenant.cache.TenantCache',
            'REAL_BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

HOBO_ROLE_EXPORT = True

SESSION_COOKIE_SECURE = False
CSRF_COOKIE_SECURE = False

LANGUAGE_CODE = 'en'

LOGGING['handlers']['debug']['filename'] = 'debug.log'
